package com.github.dakusui.jcunit8.experiments.compat;

import com.github.dakusui.jcunit.core.tuples.Tuple;
import com.github.dakusui.peerj.join.JoinReport;
import com.github.dakusui.peerj.model.FactorSpaceSpec;
import com.github.dakusui.jcunit8.factorspace.Factor;
import com.github.dakusui.jcunit8.factorspace.FactorSpace;
import com.github.dakusui.jcunit8.pipeline.Requirement;
import com.github.dakusui.jcunit8.pipeline.stages.Joiner;
import com.github.dakusui.peerj.utils.CoveringArrayGenerationUtils;
import org.junit.AfterClass;
import org.junit.Before;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;

import static com.github.dakusui.peerj.utils.JoinExperimentUtils.loadPregeneratedOrGenerateAndSaveCoveringArrayFor;
import static com.github.dakusui.jcunit8.testutils.UTUtils.configureStdIOs;
import static com.github.dakusui.peerj.utils.CoveringArrayGenerationUtils.assertCoveringArray;

public abstract class JoinExperimentBase {
  private static boolean     initialized    = false;
  private static List<Tuple> lhs            = null;
  private static FactorSpace lhsFactorSpace = null;

  @AfterClass
  public static void afterClass() {
    synchronized (JoinExperimentBase.class) {
      initialized = false;
    }
  }

  public static FactorSpaceSpec factorSpecSpec(String r, int numFactors) {
    return new FactorSpaceSpec(r).addFactors(2, numFactors);
  }

  @Before
  public void before() {
    synchronized (JoinExperimentRhs2_n.class) {
      if (!initialized) {
        configureStdIOs();
        FactorSpaceSpec lhsSpec = lhsFactorSpaceSpec();
        lhsFactorSpace = lhsSpec.toFactorSpace();
        lhs = loadPregeneratedOrGenerateAndSaveCoveringArrayFor(
            lhsSpec,
            strength(),
            CoveringArrayGenerationUtils::generateWithIpoGplus);
        assertCoveringArray(lhs, lhsFactorSpace, strength());
        initialized = true;
      }
    }
  }

  protected abstract int strength();

  protected abstract FactorSpaceSpec lhsFactorSpaceSpec();

  public void exerciseJoin(int times, FactorSpaceSpec rhsSpec, List<Tuple> rhs) {
    System.out.println(JoinReport.header());
    List<Tuple> joined = null;
    for (int i = 0; i < times; i++) {
      CoveringArrayGenerationUtils.StopWatch stopWatch = new CoveringArrayGenerationUtils.StopWatch();
      joined = exerciseJoin(rhs);
      System.out.printf(
          "%s%n",
          new JoinReport(
              Integer.toString(lhsFactorSpace.getFactorNames().size()),
              Objects.toString(rhsSpec.numFactors()),
              joined.size(),
              stopWatch.get()
          )
      );
    }
    FactorSpace joinedFactorSpace = FactorSpace.create(
        new ArrayList<Factor>(lhsFactorSpaceSpec().numFactors() + rhsSpec.numFactors()) {{
          addAll(lhsFactorSpaceSpec().toFactorSpace().getFactors());
          addAll(rhsSpec.toFactorSpace().getFactors());
        }},
        Collections.emptyList()
    );
    joined.forEach(System.out::println);
    assertCoveringArray(joined, joinedFactorSpace, strength());
  }

  private List<Tuple> exerciseJoin(List<Tuple> rhs) {
    return CoveringArrayGenerationUtils.join(
        lhs,
        rhs,
        joinerFactory(),
        strength()
    );
  }

  protected Function<Requirement, Joiner> joinerFactory() {
    return Joiner.Standard::new;
  }

}
